import sys
import numpy
from pylab import *
import math
import fabri
from time import *

def fp(r,t,beta,L):
    import sys 
    import numpy 
    import math
    T11 = 1/(t**2)*(numpy.exp(beta*L*1j)-r**2*numpy.exp(-beta*L*1j))
    T21 = -1/(t**2)*(r*numpy.exp(beta*L*1j)-r*numpy.exp(-beta*L*1j))
    T12 = 1/(t**2)*(r*numpy.exp(-beta*L*1j)-r*numpy.exp(beta*L*1j))
    T22 = 1/(t**2)*(numpy.exp(-beta*L*1j)-r**2*numpy.exp(beta*L*1j))
    T = numpy.array(((T11,T12),(T21,T22)),complex)
    return T

def seg(r,t,beta,L):
    #calculate T matrix of a segment whose length is L
    import sys 
    import numpy 
    import math
    T11 = 1/t*numpy.exp(beta*L*1j)
    T21 = r/t*numpy.exp(beta*L*1j)
    T12 = r/t*numpy.exp(-beta*L*1j)
    T22 = 1/t*numpy.exp(-beta*L*1j)
    T = numpy.array(((T11,T12),(T21,T22)),complex)
    return T

def mperiod(r,t,beta1,L1,beta2,L2,m):
    # calculate the T matrix of an M-period grating
    import sys 
    import numpy 
    import math
    phip = beta1*L1+beta2*L2
    phim = beta1*L1-beta2*L2
    T11 = 1/t**2*(numpy.exp(phip*1j)-r**2*numpy.exp(-phim*1j))
    T21 = r/t**2*(numpy.exp(phip*1j)-numpy.exp(-phim*1j))
    T12 = r/t**2*(numpy.exp(-phip*1j)-numpy.exp(phim*1j))
    T22 = 1/t**2*(numpy.exp(-phip*1j)-r**2*numpy.exp(phim*1j))

    psi = numpy.log(0.5*(T11+T22)+(0.25*(T11+T22)**2-1)**0.5)
    DELTA = 1j*(T22-T11)/(T22+T11)
    meff = numpy.tanh(m*psi)/numpy.tanh(psi)

    Tg11 = (1+1j*meff*DELTA)*numpy.cosh(m*psi)
    Tg21 = T21/T11*meff*(1+1j*DELTA)*numpy.cosh(m*psi)
    Tg12 = T12/T22*meff*(1-1j*DELTA)*numpy.cosh(m*psi)
    Tg22 = (1-1j*meff*DELTA)*numpy.cosh(m*psi)

    T = numpy.array(((Tg11,Tg12),(Tg21,Tg22)),complex)
    return T

def inrange(val,gain,delrange):
    if (val>gain-delrange) and (val<gain+delrange):
        return 1
    else:
        return 0

def clean_data(numpoints,scan_results,lam,Lg,delta_Lg,delta_gain):
    final_scan_results = numpy.zeros((numpoints,3)) #store the results of gain resonance in this array. Column 0 = max_value; Column 2 = gain
    i = 1
    print Lg
    while (i<numpoints-1):
        while ((scan_results[i,0]==0)and(i<numpoints-1)):
            i = i + 1
        local_gain = scan_results[i,1]
        maximum_ind = i
        maximum_va = scan_results[i,0]
        while (inrange(scan_results[i,1],local_gain,delta_gain) )and(i<numpoints-1):
            if (scan_results[i,0]>=maximum_va):
                maximum_va = scan_results[i,0]
                maximum_ind = i
            i = i+1
        if (i<numpoints-1):     
            final_scan_results[maximum_ind,0] = maximum_va
            final_scan_results[maximum_ind,1] = scan_results[maximum_ind,1]
            final_scan_results[maximum_ind,2] = scan_results[maximum_ind,1]*Lg
    return final_scan_results
    
def plot_data(lam,delta_Lg,final_scan_results,kappaLG,Lg,titlestr,typestr):
    figure(1)
    plot(lam,final_scan_results[:,1],'rs');
    title(titlestr)
    ylabel('Gain $cm^{-1}$',fontsize = 16)
    xlabel('Wavelength $(\mu m)$',fontsize = 16)
    grid(True)

    final_scan_results[:,1] = final_scan_results[:,1]*Lg
    figure(2)
    plot(lam,final_scan_results[:,1],'rs');
    title(titlestr)
    ylabel('$(\Gamma g - \\alpha_i)L_g$',fontsize = 16)
    xlabel('Wavelength $(\mu m)$',fontsize = 16)
    grid(True)

    figure(3)
    plot(lam,final_scan_results[:,0],'rs');
    title(titlestr)
    ylabel('Value (a.u.)',fontsize = 16)
    xlabel('Wavelength $(\mu m)$',fontsize = 16)
    grid(True)

    figure(4)
    plot(delta_Lg,final_scan_results[:,1],'rs');
    title(titlestr)
    ylabel('$(\Gamma g - \\alpha_i)L_g$',fontsize = 16)
    xlabel('$(\delta L_g)$',fontsize = 16)
    grid(True)

    show()
    return

def definitions(kappaLG,gmin,gmax):
    gnumpoints = int(round((gmax-gmin)*2))
    if gnumpoints == 0:
        gnumpoints = 1
#    gnumpoints = 21
    delta_gain = 0.25 #range of gain variations for the same solution
    lam0 = 1.550

    n1 = 3.23
    n2 = 3.2315
    n_average = (n1+n2)/2
    
    r = (n2-n1)/(n1+n2)
    t = (1-r**2)**(0.5)
    kappa = 2*(n2-n1)/lam0     #

    Lp1 = lam0/(4*n1)*10**(-4) #length of one period
    Lp2 = lam0/(4*n2)*10**(-4) #length of one period
    
    m = round(kappaLG/(2*r))   #number of periods
    Lg = (Lp1+Lp2)*m
    lrange = 1/(1/lam0-15/(2*pi*n_average*Lg*10**4))-lam0
    numpoints = int(round(lrange*1000)*501)
    lam = numpy.linspace(lam0-lrange,lam0+lrange,numpoints) #values of wavelength for the S21 calculation
    delta_Lg = 2*pi*n_average*(1/lam0 - 1/lam)*Lg*10**4

    return numpoints,gnumpoints,delta_gain,lam0,lrange,n1,n2,n_average,r,t,kappa,Lp1,Lp2,m,Lg,lam,delta_Lg

def dfb_std(kappaLG,gmin,gmax,threshold): #standard DFB with AR coating
    #load all values from the common place
    (numpoints,gnumpoints,delta_gain,lam0,lrange,n1,n2,n_average,r,t,kappa,Lp1,Lp2,m,Lg,lam,delta_Lg) = definitions(kappaLG,gmin,gmax)
    print numpoints
    gain = numpy.linspace(gmin,gmax,gnumpoints)                  #values of gain for the S21 calculation
    S21 = numpy.zeros((gnumpoints, numpoints))

    scan_results = numpy.zeros((numpoints,2)) #store the results of gain resonance in this array. Column 0 = max_value; Column 2 = gain
    final_scan_results = numpy.zeros((numpoints,3)) #store the results of gain resonance in this array. Column 0 = max_value; Column 2 = gain

    rin = 0        #ref index going from air into DFB - AR coating assumed
    tin = (1-rin**2)**(0.5)    #DFB starts with the high index material

    rout = 0       #ref index going from DFB into air. Ar coating assumed
    tout = (1-rout**2)**(0.5)  #DFB ends with the high index material

#####################################################
    for g in range (0,gnumpoints):
        status = 100.0*g/(gnumpoints)
        print 'complete =',status,'%'
        for i in range (0,numpoints):
            beta2 = 2*math.pi/lam[i]*n2*10**4+1j*gain[g]/2
            beta1 = 2*math.pi/lam[i]*n1*10**4+1j*gain[g]/2
            T1 = fabri.mperiod(-r,t,beta1,Lp1,beta2,Lp2,m)
            Tin = fabri.seg(rin,tin,beta2,Lp2)   #input segment
            K = numpy.dot(Tin,T1)
            S21[g,i] = abs(1/K[0,0])
            if S21[g,i]>threshold:
                if S21[g,i]>scan_results[i,0]:
                    scan_results[i,0] = S21[g,i]
                    scan_results[i,1] = gain[g]
                   
    titlestr='Standard DFB with AR; $\kappa L_g =$'+str(kappaLG)
    typestr ='DFB_STD'
    plot_data(lam,delta_Lg,final_scan_results,kappaLG,Lg,titlestr,typestr)
    return delta_Lg,kappaLG,final_scan_results,titlestr,typestr
    
def dfb_qtr(kappaLG,gmin,gmax,threshold): #quarter shifted DFB with AR coating
    #kappa*Lg, maximum modal gain, threshold value for the function blowup
    (numpoints,gnumpoints,delta_gain,lam0,lrange,n1,n2,n_average,r,t,kappa,Lp1,Lp2,m,Lg,lam,delta_Lg) = definitions(kappaLG,gmin,gmax)    

    rin = 0        #ref index going from air into DFB. 
    tin = (1-rin**2)**(0.5)    #DFB starts with the high index material

    rout = 0       #ref index going from DFB into air. 
    tout = (1-rout**2)**(0.5)  #DFB ends with the high index material

    gain = numpy.linspace(gmin,gmax,gnumpoints)                  #values of gain for the S21 calculation
    S21 = numpy.zeros((gnumpoints, numpoints))
    scan_results = numpy.zeros((numpoints,2)) #store the results of gain resonance in this array. Column 0 = max_value; Column 2 = gain
    final_scan_results = numpy.zeros((numpoints,3)) #store the results of gain resonance in this array. Column 0 = max_value; Column 2 = gain

    for g in range (0,gnumpoints):
        status = 100.0*g/(gnumpoints)
        print 'complete =',status,'%'
        for i in range (0,numpoints):
            beta2 = 2*math.pi/lam[i]*n2*10**4+1j*gain[g]/2
            beta1 = 2*math.pi/lam[i]*n1*10**4+1j*gain[g]/2
            T1 = fabri.mperiod(-r,t,beta1,Lp1,beta2,Lp2,round(m/2))
            Tin = fabri.seg(rin,tin,beta2,Lp2)   #input segment
            Tshift = fabri.seg(0,1,beta2,Lp2)
            K = numpy.dot(Tin,T1)
            K = numpy.dot(K,Tshift)
            K = numpy.dot(K,T1)
            S21[g,i] = abs(1/K[0,0])
            if S21[g,i]>threshold:
                if S21[g,i]>scan_results[i,0]:
                    scan_results[i,0] = S21[g,i]
                    scan_results[i,1] = gain[g]

    final_scan_results = clean_data(numpoints,scan_results,lam,Lg,delta_Lg,delta_gain,fname)
    titlestr='$\lambda/4$ shifted DFB with AR; $\kappa L_g =$'+str(kappaLG)
    typestr ='DFB_QTR'
#    plot_data(lam,delta_Lg,final_scan_results,kappaLG,Lg,titlestr,typestr)
    return delta_Lg,kappaLG,final_scan_results,titlestr,typestr

########################################################################
#MAIN PROGRAM
########################################################################

begin = 60
ende = 140 
kLg = 1.0
numpoints = 1

for i in range (0,numpoints):
    (delta_Lg,kappaLG,final_scan_results,titlestr,typestr) = fabri.dfb_std(kLg,begin,ende,100) 
