% This programs solves for the 2 mode cutoff waveguide width of a burried ridge waveguide
% based on the etched depth into the waveguide.
% This program solves for the effective index of the slab waveguide. It is based on 
% the paper by H Kogelnik and V Ramaswamy, August 1974, Vol 13, No 8, Applied Optics, 
% page 1857 Copyright Milan Masanovic, January 2001

% define the waveguide parameters
%         =================
%         ===== Nc ========
%         =================
%         /////////////////  |
%         ///// Nf ////////  | d
%         /////////////////  |
%         \\\\\\\\\\\\\\\\\
%         \\\\\ Ns \\\\\\\\
%         \\\\\\\\\\\\\\\\\
% define V,a,m as global variables
clear all;
global V a m
% ! WARNING - Ns ALWAYS has to be greater that Nc !
Nc = 3.2322;
Nf = 3.2920;
Ns = 3.2322;
% free space wavelength (microns)
lambda = 1.55;
% waveguide width (microns)
w_thick = 0.35;
d1 = 0.4:0.01:0.7;
len = length(d1)
cutoff = zeros(1,len);
cutoff2 = zeros(1,len);
n_modes = zeros(1,len);
for j = 1:len
   Nc = 1;
   Nf = 3.3865;
   Ns = 1;
   % mode number 
   m = 0;
   % free space wave vector
   k = 2.*pi./lambda;
   % normalized waveguide thickness
   V = k.*d1(j).*sqrt(Nf^2-Ns^2);
   % asymmetry measure
   a = (Ns^2-Nc^2)/(Nf^2-Ns^2);
   % calculate all the possible values for V vs b combination
   b = 0:0.0005:0.9990;
   V1 = 1./sqrt(1-b).*(m.*pi + atan(sqrt(b./(1-b)))+atan(sqrt((b+a)./(1-b))));
   % find a value of b close to V
   %len = length(b);
   i = 1;
   while V1(i) < V
      i = i+1;
   end
   % calculate the actual b and N_eff
   answer = fzero('f',b(i));
   N_eff = sqrt(Ns^2+answer.*(Nf^2-Ns^2))
   milan = d1(j);
   Nc = N_eff;
   Ns = N_eff;
   Nf = 3.2887;
   a = (Ns^2-Nc^2)/(Nf^2-Ns^2);
   % calculate all the possible values for V vs b combination
   V_cut = pi;
   V_cut2 = 2.*pi;
   cutoff(j) = V_cut/(k.*sqrt(Nf^2-Ns^2));
   cutoff2(j) = V_cut2/(k.*sqrt(Nf^2-Ns^2));
   % total number of the modes
   n_modes(j) = floor((V-atan(sqrt(a)))./pi)+1;
end   
etch_depth = w_thick-d1;
% Plot the graph and the solution, together with parameter values
figure(1);
plot(etch_depth,cutoff,'r',etch_depth,cutoff2,'b');
grid on;
title(strcat('Maximum width for single mode waveguide vs. etch depth (wgd thickness 0.35 um)'));
legend('Mode 0 Cutoff','Mode 1 Cutoff');
xlabel('Waveguide Width (um)');
ylabel('Etch Depth (um)');
figure(2);
plot(etch_depth,cutoff,'r',etch_depth,cutoff2,'b');
legend('Mode 0 Cutoff','Mode 1 Cutoff');
grid on;
title(strcat('Maximum width for single mode waveguide vs. etch depth (wgd thickness 0.35 um)'));
xlabel('Etch Depth (um)');
ylabel('Waveguide Width (um)');
axis([0 0.35 0 6]);