function [x3,y3,mode3,neff3] = WaveG2D(guide,lambda,order)

k0 = 2*pi/lambda;


%******************************************************************
% generate a uniform 22x22 matrix
%******************************************************************

YW = sum(guide.thick.vert);
XW = sum(guide.thick.horz);

N = 22;
M = 22;

[x1,y1] = meshgrid([-XW/2:XW/(M+1):XW/2],[-YW/2:YW/(M+1):YW/2]);
[n1] = WaveIndex2D(x1,y1,guide);

%******************************************************************
% solve the uniform mesh with a 22x22 matrix using eigs (fd22new)
%******************************************************************

MN=M*N;

% Remove the boundary points and create the middle diagonal
% We need N terms for each dxi minus dxi for i = 2:N+1 M times 
dxi = reshape(diff(x1(2:N+1,2:M+2)')', MN, 1 );
dxi_1 = reshape(diff(x1(2:N+1,1:M+1)')', MN, 1 );
DXi = dxi*k0;
DXi_1 = dxi_1*k0;

dyj = reshape(diff(y1(2:N+2,2:M+1)), MN, 1 );
dyj_1 = reshape(diff(y1(1:N+1,2:M+1)), MN, 1 );
DYj = dyj*k0;
DYj_1 = dyj_1*k0;

% Generate the 0 diagonal for the A matrix
a = reshape(n1(2:N+1,2:M+1).^2 , MN ,1 );
a = a - 2*(1./(DXi.*DXi_1) + 1./(DYj.*DYj_1));

% Generate the +1 and -1 diagonals for the A matrix
b = 2./(DYj_1.*(DYj+DYj_1));
c = 2./(DYj.*(DYj+DYj_1));

% Generate the -N and +N diagonals for the A matrix
d = 2./(DXi_1.*(DXi+DXi_1));
e = 2./(DXi.*(DXi+DXi_1));

% Generate the A matrix as a sparse matrix using the digonals
c(2:MN) = c(1:MN-1);
e(N+1:MN) = e(1:(M-1)*N);
A = spdiags([a,c,e],[0,1,N],MN,MN);
A = spdiags(b(2:MN),-1,A);
A = spdiags(d(N+1:MN),-N,A);

% To help with memory requirements, clear some variables
clear dxi dxi_1 DXi DXi_1 dyj dyj_1 DYj DYj_1 a b c d e

% Zero out every N+1 term on the -1 and +1 diagonal 
for ii = 1 : M-1;
	A((ii*N) + 1 , ii*N) = 0;
	A( ii*N , (ii*N) + 1) = 0;
end

% Set the options for the function eigs and solve for the eigenvalues and
% eigenvectors for the selected mode.  Then, sort the eigenvalues so that
% the largest one can be chosen
options.disp = 0;
[P,R] = eigs(A,eye(MN),1+order,'LR',options);
[q,k] = sort(real(diag(R)));
ind = length(q);

clear A

% The effective refractive index is the square root of the eigenvalue
neff1 = sqrt(q(ind-order));
vvv = P(:,k);
mode1 = vvv(:,ind-order);
mode11 = reshape(mode1,N ,M);

% Place zeros around the mode to include the boundary conditions
mode1 = zeros(N+2, M+2);
mode1(2: N+1, 2: M+1) = mode11;

%Normalize so that the mode is 1 at its maximum point
mode1 = mode1 / max(max(abs(mode1)));

clear modevvv mode11 P


%******************************************************************
% interpolate mode1 onto a user defined grid
%******************************************************************

[x2,y2] = WaveData2D(guide);
mode2 = interp2(x1,y1,mode1,x2,y2,'cubic');

mode2 = mode2 / max(max(abs(mode2)));
mode2 = excise(mode2,0,0);


%******************************************************************
% compute a non-uniform grid based on slope of mode
%******************************************************************

vertp = [0,cumsum(guide.point.vert)];
horzp = [0,cumsum(guide.point.horz)];
dyf = [];
dxf = [];

for i = 1:length(guide.thick.vert)
   ys = [vertp(i)+1:vertp(i+1)];
   dy = diff(y2(ys,:))/1e4;
	slopey = max([abs(diff(mode2(ys,:)))./dy]')+1;

%   this line is what James wrote
%	cy = fzero('gridconst',[1e-3 10],1e-7,[],(slopey),abs(sum(dy(:,1))));
	cy = fzero('gridconst',[1e-3 10], options, slopey, abs(sum(dy(:,1))));

    dyf = [dyf,.1,(cy.^slopey)*1e4];
end


for i = 1:length(guide.point.horz)
   xs = [horzp(i)+1:horzp(i+1)];
   dx = diff(x2(:,xs)')/1e4;
	slopex = max([abs(diff(mode2(:,xs)')./dx)]')+1;
    
%   this line is what James wrote
%	cx = fzero('gridconst',[1e-3 10],1e-7,[],slopex,abs(sum(dx(:,1))));
	cx = fzero('gridconst',[1e-3 10], options,slopex,abs(sum(dx(:,1))));
    dxf = [dxf,.1,(cx.^slopex)*1e4];
end

dyf(1) = 0;
dxf(1) = 0;
y3 = cumsum(dyf)-sum(guide.thick.vert)/2;
x3 = cumsum(dxf)-sum(guide.thick.horz)/2;

y3(length(y3)) = sum(guide.thick.vert)/2;
x3(length(x3)) = sum(guide.thick.horz)/2;

[x3,y3] = meshgrid(x3,y3);

mode3 = interp2(x2,y2,mode2,x3,y3,'cubic');

mode3 = mode3 / max(max(mode3));
mode3 = excise(mode3,0,0);

[n3] = WaveIndex2D(x3,y3,guide);

M = sum(guide.point.horz)-2;
N = sum(guide.point.vert)-2;


%******************************************************************
% inpower3
%******************************************************************

MN=M*N;

% Remove the boundary points and create the middle diagonal
% We need N terms for each dxi minus dxi for i = 2:N+1 M times 
dxi = reshape(diff(x3(2:N+1,2:M+2)')', MN, 1 );
dxi_1 = reshape(diff(x3(2:N+1,1:M+1)')', MN, 1 );
DXi = dxi*k0;
DXi_1 = dxi_1*k0;

dyj = reshape(diff(y3(2:N+2,2:M+1)), MN, 1 );
dyj_1 = reshape(diff(y3(1:N+1,2:M+1)), MN, 1 );
DYj = dyj*k0;
DYj_1 = dyj_1*k0;

% Generate the 0 diagonal for the A matrix
a = reshape(n3(2:N+1,2:M+1).^2 , MN ,1 );
a = a - 2*(1./(DXi.*DXi_1) + 1./(DYj.*DYj_1));

% Generate the +1 and -1 diagonals for the A matrix
b = 2./(DYj_1.*(DYj+DYj_1));
c = 2./(DYj.*(DYj+DYj_1));

% Generate the -N and +N diagonals for the A matrix
d = 2./(DXi_1.*(DXi+DXi_1));
e = 2./(DXi.*(DXi+DXi_1));

% Generate the A matrix as a sparse matrix using the digonals
c(2:MN) = c(1:MN-1);
e(N+1:MN) = e(1:(M-1)*N);
A = spdiags([a,c,e],[0,1,N],MN,MN);
A = spdiags(b(2:MN),-1,A);
A = spdiags(d(N+1:MN),-N,A);

% To help with memory requirements, clear some variables
clear dxi dxi_1 DXi DXi_1 dyj dyj_1 DYj DYj_1 a b c d e

% Zero out every N+1 term on the -1 and +1 diagonal 
for ii = 1 : M-1;
	A((ii*N) + 1 , ii*N) = 0;
	A( ii*N , (ii*N) + 1) = 0;
end

iter = 3;
beta(1,1) = neff1^2;
test1= mode3(2:N+1, 2:M+1);
Xx(:,1) = reshape(test1, MN,1);

% This is the code to make the A matrix Hermitian which allows the 
% eigenvectors and eigenvalues to be solved faster and more accurately
%D = sqrt(spdiags((dxi+dxi_1).*(dyj+dyj_1),0,MN,MN));
%A = inv(D)*D^2*A*inv(D);
%Xx(:,1) = D*Xx(:,1);

% Iterate on the mode shape and effective index iter times
for ii = 1:iter ;
	Xx(:,ii+1) = (A - (beta(1,ii) * speye(MN))) \ Xx(:,ii) ;
	Xx(:,ii+1) = (Xx(:,ii+1)) / ((Xx(:,ii+1))' * Xx(:,ii+1));
	beta(1,ii+1) = ((Xx(:,ii+1))' * A * (Xx(:,ii+1) )/((Xx(:,ii+1))' * Xx(:,ii+1) ));
	neff3 = sqrt(beta(1, ii+1));  
end

neff3 = sqrt(beta(1, ii+1));
mode3 = Xx(:, ii+1);
mode33 = reshape(mode3,N ,M);

% Place zeros around the mode to include the boundary conditions
mode3 = zeros(N+2, M+2);
mode3(2: N+1, 2: M+1) = mode33;

%Normalize so that the mode is 1 at its maximum point
mode3 = mode3 / max(max(abs(mode3)));
clear Xx




