import sys
import numpy
from pylab import *
import math
from time import *
import matplotlib.pyplot as plt

def fp(r,t,beta,L):
    import sys 
    import numpy 
    import math
    T11 = 1/(t**2)*(numpy.exp(beta*L*1j)-r**2*numpy.exp(-beta*L*1j))
    T21 = -1/(t**2)*(r*numpy.exp(beta*L*1j)-r*numpy.exp(-beta*L*1j))
    T12 = 1/(t**2)*(r*numpy.exp(-beta*L*1j)-r*numpy.exp(beta*L*1j))
    T22 = 1/(t**2)*(numpy.exp(-beta*L*1j)-r**2*numpy.exp(beta*L*1j))
    T = numpy.array(((T11,T12),(T21,T22)),complex)
    return T

def seg(r,t,beta,L):
    #calculate T matrix of a segment whose length is L
    import sys 
    import numpy 
    import math
    T11 = 1/t*numpy.exp(beta*L*1j)
    T21 = r/t*numpy.exp(beta*L*1j)
    T12 = r/t*numpy.exp(-beta*L*1j)
    T22 = 1/t*numpy.exp(-beta*L*1j)
    T = numpy.array(((T11,T12),(T21,T22)),complex)
    return T

def mperiod(r,t,beta1,L1,beta2,L2,m):
    # calculate the T matrix of an M-period grating
    import sys 
    import numpy 
    import math
    phip = beta1*L1+beta2*L2
    phim = beta1*L1-beta2*L2
    T11 = 1/t**2*(numpy.exp(phip*1j)-r**2*numpy.exp(-phim*1j))
    T21 = r/t**2*(numpy.exp(phip*1j)-numpy.exp(-phim*1j))
    T12 = r/t**2*(numpy.exp(-phip*1j)-numpy.exp(phim*1j))
    T22 = 1/t**2*(numpy.exp(-phip*1j)-r**2*numpy.exp(phim*1j))

    psi = numpy.log(0.5*(T11+T22)+(0.25*(T11+T22)**2-1)**0.5)
    DELTA = 1j*(T22-T11)/(T22+T11)
    meff = numpy.tanh(m*psi)/numpy.tanh(psi)

    Tg11 = (1+1j*meff*DELTA)*numpy.cosh(m*psi)
    Tg21 = T21/T11*meff*(1+1j*DELTA)*numpy.cosh(m*psi)
    Tg12 = T12/T22*meff*(1-1j*DELTA)*numpy.cosh(m*psi)
    Tg22 = (1-1j*meff*DELTA)*numpy.cosh(m*psi)

    T = numpy.array(((Tg11,Tg12),(Tg21,Tg22)),complex)
    return T



##################################################
lam0 = 1.550

len_grat = 0.2394

n1 = 3.22478
n2 = 3.248

z0 = 68.5*10**(-4)
z1 = 4*10**(-4)
n_bursts = 5

loss = 9

    
r = (n2-n1)/(n1+n2)
t = (1-r**2)**(0.5)
kappa = 2*(n2-n1)/lam0     #


Lp1 = len_grat*10**(-4)/2
Lp2 = len_grat*10**(-4)/2

#Lp1 = lam0/(4*n1)*10**(-4) #length of one period
#Lp2 = lam0/(4*n2)*10**(-4) #length of one period

    
m = round(z1/(0.235*10**(-4)))   #number of periods
Lg = (Lp1+Lp2)*m
numpoints = 10000
lam = numpy.linspace(1.450,1.650,numpoints) #values of wavelength for the S21 calculation


#################################

rin = 0
tin = 1
rout = 0.0      
tout = (1-rout**2)**(0.5)  
 
S11 = numpy.zeros(numpoints)

for i in range (0,numpoints):
    beta2 = 2*math.pi/lam[i]*n2*10**4-1j*loss/2
    beta1 = 2*math.pi/lam[i]*n1*10**4-1j*loss/2
    T1 = mperiod(-r,t,beta1,Lp1,beta2,Lp2,m)
    Tseg1 = seg(0,1,beta1,(z0-z1))   #input segment, AR coated
    L = numpy.dot(T1,Tseg1)
    K = L
    for j in range (0,n_bursts-2):
        K = numpy.dot(K,L)
    S11[i] = abs(K[1,0]/K[0,0])**2

SFM = S11

z0 = 61.5*10**(-4)
z1 = 6*10**(-4)
n_bursts = 10
m = round(z1/(0.235*10**(-4)))   #number of periods

S11 = numpy.zeros(numpoints)

for i in range (0,numpoints):
    beta2 = 2*math.pi/lam[i]*n2*10**4-1j*loss/2
    beta1 = 2*math.pi/lam[i]*n1*10**4-1j*loss/2
    T1 = mperiod(-r,t,beta1,Lp1,beta2,Lp2,m)
    Tseg1 = seg(0,1,beta1,(z0-z1))   #input segment, AR coated
    L = numpy.dot(T1,Tseg1)
    K = L
    for j in range (0,n_bursts-2):
        K = numpy.dot(K,L)
    S11[i] = abs(K[1,0]/K[0,0])**2

SBM = S11

lam = lam*1000

fig = figure(facecolor ='white')
ax = fig.add_subplot(111)

ylabel('Reflectivity',fontsize = 20)
xlabel('Wavelength (nm)',fontsize = 20)

ax.plot(lam,SFM,'k',linewidth = 1.5)
ax.plot(lam,SBM,'k:',linewidth = 1.5)
ax.grid('on')
ax.set_xlim(1500,1600)
ax.set_ylim (0,1.0)

ax.axis([1500,1600,0,1])

for tl in ax.get_yticklabels():
    tl.set_fontsize(20)

for tl in ax.get_xticklabels():
    tl.set_fontsize(20)


show()
