% This example shows how to calculate and plot both the
% fundamental quasi-TE and quasi-TM eigenmodes of an example
% 3-layer ridge waveguide using the semivectorial eigenmode
% solver.  

% Refractive indices:
n1 = 3.34;          % Lower cladding
n2 = 3.44;          % Core
n3 = 1.00;          % Upper cladding (air)

% Layer heights:
h1 = 2.0;           % Lower cladding
h2 = 1.3;           % Core thickness
h3 = 0.5;           % Upper cladding

rh = 1.1;           % Ridge height
rw = 1.0;           % Ridge half-width
side = 1.5;         % Space on side

dx = 0.0125;        % grid size (horizontal)
dy = 0.0125;        % grid size (vertical)

lambda = 1.55;      % vacuum wavelength
nmodes = 1;         % number of modes to compute
boundary = '000S';  % boundary condition

fprintf (1,'generating index mesh...\n');
[x,y,xc,yc,nx,ny,eps] = waveguidemesh([n1,n2,n3],[h1,h2,h3], ...
                                      rh,rw,side,dx,dy); 

fprintf (1,'solving for eigenmodes (TE) ... '); t = cputime;
[ExTE,neffTE] = svmodes (lambda, n2, nmodes, dx, dy, eps, ...
                         boundary, 'EX'); 
fprintf (1,'done (cputime = %7.3f)\n', cputime-t);
fprintf(1,'neff(TE) = %7.5f\n',neffTE);

fprintf (1,'solving for eigenmodes (TM) ... '); t = cputime;
[EyTM,neffTM] = svmodes (lambda, n2, nmodes, dx, dy, eps, ...
                         boundary, 'EY'); 
fprintf (1,'done (cputime = %7.3f)\n', cputime-t);
fprintf(1,'neff(TM) = %7.5f\n',neffTM);

subplot(121);
contourmode(xc,yc,ExTE,1,3,45,'Ex (TE mode)');
v = xlim;
line(v,[h1,h1]);
line([v(1),rw,rw,v(2)],[h1+h2,h1+h2,h1+h2-rh,h1+h2-rh]);

subplot(122);
contourmode(xc,yc,EyTM,1,3,45,'Ey (TM mode)');
line(v,[h1,h1]);
line([v(1),rw,rw,v(2)],[h1+h2,h1+h2,h1+h2-rh,h1+h2-rh]);
